/*
*  PopAccDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
#define _WINSOCKAPI_
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include <assert.h>

#include "PopAccDlgs.h"
#include "PopProfDlg.h"
#include "PopProxyDlgs.h"
#include "PopRetDlg.h"
#include "Fileman.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TPopAcctDlg2 *PopAcctDlg2;
//---------------------------------------------------------------------------

bool __fastcall OpenPopAccountsDlg(TForm *Owner, TStrings *accts)
{
   PopAcctDlg2 = new TPopAcctDlg2(Owner);
   PopAcctDlg2->Accts->Items->Text = accts->Text;
   PopAcctDlg2->ShowModal();
   bool ok = PopAcctDlg2->ModalResult == mrOk;
   if (ok)
      accts->Text = PopAcctDlg2->Accts->Items->Text;
   delete PopAcctDlg2;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TPopAcctDlg2::TPopAcctDlg2(TComponent* Owner)
        : TForm(Owner)
{
   AnsiString proxy;
   CenterDialog(this, Owner);

   TStringList *proxies = new TStringList();
   GetAppData(proxies, "proxies");
   ProxyBox->Items->Add("<none>");
   ProxyBox->Height = 23;

   for (int i = 0; i < proxies->Count; i++)
   {
      AnsiString line = proxies->Strings[i];
      if (GetToken(3, line, ",") != " ")
         ProxyBox->Items->Add(line);
   }

   proxy = GetAppIni("popproxy", "<none>");
   int idx = ProxyBox->Items->IndexOf(proxy);

   if (proxy != "" && proxy != "<none>" && idx < 0)
   {
      MessageBox(Handle,
         AnsiString("The socks proxy you have been using, '" + GetToken(0, proxy, ",") + "', has been deleted!").c_str(),
        "Warning", MB_OK);
   }

   ProxyBox->ItemIndex = idx;
   if (ProxyBox->ItemIndex < 0)
      ProxyBox->ItemIndex = 0;

   delete proxies;

   Modified = false;
}
//---------------------------------------------------------------------------

__fastcall TPopAcctDlg2::~TPopAcctDlg2()
{
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Modified)
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgDlg(this,
            "Your POP account data is modified. Save your changes?   ",
            "Confirm", MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }
      PutAppData(PopAccounts, Accts->Items);
   }
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::NewBtnClick(TObject *Sender)
{
   AnsiString profile = NewAcctProfile(this);
   if (!profile.IsEmpty())
   {
      Accts->Items->Add(profile);
      Accts->ItemIndex = Accts->Items->IndexOf(profile);
      Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::EditBtnClick(TObject *Sender)
{
   int Index = Accts->ItemIndex;
   assert(Index >= 0);

   AnsiString profile = EditAcctProfile(this, Accts->Items->Strings[Index]);
   if (!profile.IsEmpty())
   {
      Accts->Items->Strings[Index] = profile;
      Accts->ItemIndex = Index;
      ActiveControl = Accts;
      Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::DeleteBtnClick(TObject *Sender)
{
   int Index = Accts->ItemIndex;
   assert(Index >= 0);

   if (MsgDlg(this,"Delete '"+Account(Index)+"' ?    \n\n Are you sure?",
      "Delete Account", MB_OKCANCEL) == IDOK)
   {
      Accts->Items->Delete(Index);
      Modified = true;
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPopAcctDlg2::Get(int Index, const AnsiString &token)
{
   TStringList *tmp = new TStringList();
   tmp->CommaText = Accts->Items->Strings[Index];
   AnsiString item = tmp->Values[token];
   delete tmp;

   return item;
}
//---------------------------------------------------------------------------
// only for true/false fields
AnsiString __fastcall TPopAcctDlg2::Opposite(int Index, const AnsiString &token)
{
   TStringList *tmp = new TStringList();
   tmp->CommaText = Accts->Items->Strings[Index];
   AnsiString value = tmp->Values[token];
   delete tmp;

   if (value == "1") value = "0";
   else value = "1";

   return value;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::Set(int Index, const AnsiString &token, const AnsiString &value)
{
   TStringList *tmp = new TStringList();
   tmp->CommaText = Accts->Items->Strings[Index];
   tmp->Values[token] = value;
   Accts->Items->Strings[Index] = tmp->CommaText;
   Accts->ItemIndex = Index;
   delete tmp;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPopAcctDlg2::Account(int Index)
{
   return Get(Index, "account");
}
//---------------------------------------------------------------------------

bool __fastcall TPopAcctDlg2::Checked(int Index)
{
   return Get(Index, "checked") == "1";
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::GetBtnClick(TObject *Sender)
{
   TStringList *accts = new TStringList();

   for (int i = 0; i < Accts->Items->Count; i++)
      if (Checked(i))
         accts->Add(Accts->Items->Strings[i]);

   if (accts->Count > 0)
      PopRetrieveMail(accts, this, Application->MainForm, GetEveryLast->Checked);
   else
   {
      MsgBoxHwnd(Handle,
         "No accounts are marked for retreive!",
         "QuickSilver",
          MB_OK
      );
   }

   delete accts;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::ProxyBtnClick(TObject *Sender)
{
   TStringList *proxy = new TStringList();
   proxy->CommaText = GetAppIni("popproxy", "");
   if (OpenPOPProxyDlg(this, proxy))
   {
      PutAppIni("popproxy", proxy->CommaText);
   }
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::DrawCheckBox(TCanvas *Canvas, TRect &Rect, bool selected)
{
   TRect box; box.Left = Rect.Left + 2; box.Top = Rect.Top + 2;
   box.Right = box.Left + 12; box.Bottom = box.Top + 12;
   Canvas->Brush->Color = selected ? clWindow : clGray;
   Canvas->FrameRect(box);
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::DrawCheck(TCanvas *Canvas, TRect &Rect, bool selected)
{
   TColor color = Canvas->Pen->Color;
   Canvas->Pen->Color = selected ? clWindow : clHighlight;
   int x = 2, y = 2;

   Canvas->MoveTo(Rect.Left + x +  2, Rect.Top + y + 5);
   Canvas->LineTo(Rect.Left + x +  4, Rect.Top + y + 7);
   Canvas->LineTo(Rect.Left + x + 10, Rect.Top + y + 1);

   Canvas->MoveTo(Rect.Left + x +  2, Rect.Top + x + 6);
   Canvas->LineTo(Rect.Left + x +  4, Rect.Top + x + 8);
   Canvas->LineTo(Rect.Left + x + 10, Rect.Top + x + 2);

   Canvas->MoveTo(Rect.Left + x +  2, Rect.Top + y + 7);
   Canvas->LineTo(Rect.Left + x +  4, Rect.Top + y + 9);
   Canvas->LineTo(Rect.Left + x + 10, Rect.Top + y + 3);

   Canvas->Pen->Color = color;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::AcctsDrawItem(TWinControl *Control, int Index,
        TRect &Rect, TOwnerDrawState State)
{
   TRect r = Rect;
   AnsiString text;
   TCanvas *canvas = Accts->Canvas;
   TColor clBr = canvas->Brush->Color;
   bool selected = State.Contains(odSelected);
   canvas->FillRect(Rect);

   TStringList *acct = new TStringList();
   acct->CommaText = Accts->Items->Strings[Index];

   DrawCheckBox(canvas, Rect, selected);
   if (acct->Values["checked"] == "1")
      DrawCheck(canvas, Rect, selected);

   r.Left = 29; r.Right = r.Left + 3; r.Top += 7; r.Bottom = r.Top + 3;
   canvas->Brush->Color = acct->Values["remember"] == "1" ? clRed : selected ? clHighlight : clWindow;
   canvas->FillRect(r);

   r.Left += 11; r.Right += 11;
   canvas->Brush->Color = acct->Values["delete"] == "1"   ? clRed : selected ? clHighlight : clWindow;
   canvas->FillRect(r);

   r.Left += 11; r.Right += 11;
   canvas->Brush->Color = acct->Values["pgponly"] == "1"  ? clRed : selected ? clHighlight : clWindow;
   canvas->FillRect(r);

   canvas->Brush->Color = clBr;

   Rect.Left = 66;
   canvas->TextRect(Rect, Rect.Left, Rect.Top, acct->Values["account"]);

   delete acct;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::AcctsMouseDown(TObject *Sender, TMouseButton Button,
   TShiftState Shift, int X, int Y)
{
   LastMouse.x = X; LastMouse.y = Y;
   if (X >= 69) // not an area for special processing
      return;

   int Index = Accts->ItemIndex;
   TRect r = Accts->ItemRect(Index);
   if (Y < r.Top || Y > r.Bottom) // not this row
      return;

   //SetRedraw(this, 0);
   if (X >= 0 && X < 24)
   {
      Set(Index, "checked", Opposite(Index, "checked"));
      r.Left = 0; r.Right = 23;
      Modified = true;
   }
   else
   if (X >= 24 && X < 36)
   {
      if (Get(Index, "remember") == "1")
      {
         if (MsgBoxHwnd(Handle, "Forget password now?   ", "Password", MB_OKCANCEL) == IDOK)
         {
            Set(Index, "remember", "0");
            Set(Index, "password", "");
            Modified = true;
         }
      }
      else
      {
         MsgBoxHwnd(Handle, "No password to remember!   ", "Password", MB_OK);
      }
      r.Left = 24; r.Right = 35;
   }
   else
   if (X >= 36 && X < 48)
   {
      Set(Index,"delete", Opposite(Index, "delete"));
      r.Left = 36; r.Right = 47;
      Modified = true;
   }
   else
   if (X >= 48 && X < 60)
   {
      Set(Index, "pgponly", Opposite(Index, "pgponly"));
      r.Left = 48; r.Right = 59;
      Modified = true;
   }

   Accts->ItemIndex = Index;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::AcctsClick(TObject *Sender)
{
   EditBtn->Enabled = Accts->ItemIndex >= 0;
   DeleteBtn->Enabled = EditBtn->Enabled;
   AcctUp->Enabled = Accts->Items->Count > 0 && Accts->ItemIndex > 0;
   AcctDn->Enabled = Accts->Items->Count > 0 && Accts->ItemIndex < Accts->Items->Count - 1;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::PopGetThisItemClick(TObject *Sender)
{
   int Index = Accts->ItemIndex;
   if (Index < 0)
      return;

   TStringList *acct = new TStringList();
   acct->Add(Accts->Items->Strings[Index]);
   PopRetrieveMail(acct, this, Application->MainForm, GetEveryLast->Checked);
   delete acct;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::PopupMenuPopup(TObject *Sender)
{
   int Count = CheckCount(), Index = Accts->ItemIndex;
   PopGetMailItem->Enabled = Count > 0;
   PopGetThisItem->Enabled = Index >= 0;
   PopToggleItem->Enabled = Index >= 0;
   PopDeleteItem->Enabled = Index >= 0;
   PopOpenItem->Enabled = Index >= 0;
   if (Index >= 0)
   {
      if (Checked(Index)) PopToggleItem->Caption = "&Mark account";
      else PopToggleItem->Caption = "Un&mark account";
   }
}
//---------------------------------------------------------------------------

int  __fastcall TPopAcctDlg2::CheckCount()
{
   int c = 0;
   for (int i = 0; i<Accts->Items->Count; i++)
      if (Checked(i))
          c++;
   return c;
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::PopToggleItemClick(TObject *Sender)
{
   int Index = Accts->ItemIndex;
   Set(Index, "checked", Opposite(Index, "checked"));
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::ProxyBoxDrawItem(TWinControl *Control, int Index,
        TRect &Rect, TOwnerDrawState State)
{
   TCanvas *Canvas = ProxyBox->Canvas;
   Canvas->TextRect(Rect, Rect.Left + 2, Rect.Top + 3,
      GetToken(0, ProxyBox->Items->Strings[Index], ","));
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::ProxyBoxChange(TObject *Sender)
{
   PutAppIni("popproxy", ProxyBox->Items->Strings[ProxyBox->ItemIndex]);
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::AcctUpClick(TObject *Sender)
{
   int Index = Accts->ItemIndex;
   if (Index > 0)
   {
      Accts->Items->Move(Index, Index - 1);
      Accts->ItemIndex = Index - 1;
      AcctDn->Enabled = true;
   }
   else MyBeep(MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TPopAcctDlg2::AcctDnClick(TObject *Sender)
{
   int Index = Accts->ItemIndex;
   if (Index < Accts->Items->Count - 1)
   {
      Accts->Items->Move(Index, Index + 1);
      Accts->ItemIndex = Index + 1;
      AcctUp->Enabled = true;
   }
   else MyBeep(MB_OK);
}
//---------------------------------------------------------------------------
